@@ -25,6 +25,28 @@ static int family = AF_INET;
 
 #ifdef	EXOSIP_API4
 
+bool voip::publish(voip::context_t ctx, const char *uri, const char *contact, const char *event, const char *duration, const char *type, const char *body)
+{
+    bool result = true;
+    voip::msg_t msg = NULL;
+
+    eXosip_lock(ctx);
+    eXosip_build_publish(ctx, &msg, uri, contact, NULL, event, duration, type, body);
+    if(msg)
+        eXosip_publish(ctx, msg, uri);
+    else
+        result = false;
+    eXosip_unlock(ctx);
+    return result;
+}
+
+void voip::call_reference(context_t ctx, voip::call_t cid, void *route)
+{
+    eXosip_lock(ctx);
+    eXosip_call_set_reference(ctx, cid, route);
+    eXosip_unlock(ctx);
+}
+
 void voip::add_authentication(context_t ctx, const char *user, const char *secret, const char *realm, bool automatic) 
 {
     eXosip_lock(ctx);
@@ -34,6 +56,13 @@ void voip::add_authentication(context_t ctx, const char *user, const char *secre
     eXosip_unlock(ctx);
 }
 
+void voip::free_message_request(context_t ctx, msg_t msg)
+{
+    if(msg)
+        osip_message_free(msg);
+    eXosip_unlock(ctx);
+}
+
 bool voip::make_request_message(context_t ctx, const char *method, const char *to, const char *from, msg_t *msg, const char *route)
 {
     if(!msg)
@@ -416,6 +445,28 @@ void voip::release(context_t ctx)
 
 static bool active = false;
 
+bool voip::publish(voip::context_t ctx, const char *uri, const char *contact, const char *event, const char *duration, const char *type, const char *body)
+{
+    bool result = true;
+    voip::msg_t msg = NULL;
+
+    eXosip_lock();
+    eXosip_build_publish(&msg, uri, contact, NULL, event, duration, type, body);
+    if(msg)
+        eXosip_publish(msg, uri);
+    else
+        result = false;
+    eXosip_unlock();
+    return result;
+}
+
+void voip::call_reference(context_t ctx, voip::call_t cid, const char *route)
+{
+    eXosip_lock();
+    eXosip_call_set_reference(cid, route);
+    eXosip_unlock();
+}
+
 void voip::add_authentication(context_t ctx, const char *user, const char *secret, const char *realm, bool automatic) 
 {
     eXosip_lock();
@@ -425,6 +476,13 @@ void voip::add_authentication(context_t ctx, const char *user, const char *secre
     eXosip_unlock();
 }
 
+void voip::free_message_request(context_t ctx, msg_t msg)
+{
+    if(msg)
+        osip_message_free(msg);
+    eXosip_unlock();
+}
+
 bool voip::make_request_message(context_t ctx, const char *method, const char *to, const char *from, msg_t *msg, const char *route)
 {
     if(!msg)
@@ -849,3 +907,22 @@ void voip::server_supports(voip::msg_t msg, const char *txt)
     osip_message_set_supported(msg, txt);
 }
 
+void voip::header(voip::msg_t msg, const char *key, const char *value)
+{
+    osip_message_set_header(msg, key, value);
+}
+
+void voip::attach(msg_t msg, const char *type, const char *body, size_t size)
+{
+    if(body)
+        osip_message_set_body(msg, body, size);
+    if(type)
+        osip_message_set_content_type(msg, type);
+}
+
+void voip::attach(msg_t msg, const char *type, const char *body)
+{
+    attach(msg, type, body, strlen(body));
+}
+
+
