@@ -53,7 +53,8 @@
 #define USER_PROFILE_ADMIN          0xffff
 #define USER_PROFILE_LOCAL          0x0fff
 #define USER_PROFILE_DEVICE         0x0f0f
-#define USER_PROFILE_SERVICE        0xff0f
+#define USER_PROFILE_SERVICE        0x7f0f
+#define USER_PROFILE_SYSTEM         0x2f08
 #define USER_PROFILE_DEFAULT        0x7fff
 #define USER_PROFILE_RESTRICTED (0)
 
@@ -120,17 +121,17 @@ public:
         } internal;
     } source;
 
-    inline bool isUser(void) const
+    inline bool is_user(void) const
         {return (type == USER);};
 
-    inline bool isProfiled(void) const
+    inline bool is_profiled(void) const
         {return (type == USER || type == SERVICE);};
 
-    inline bool isAdmin(void) const
-        {return (isProfiled() && (profile.features & USER_PROFILE_SUPERUSER));};
+    inline bool is_admin(void) const
+        {return (is_profiled() && (profile.features & USER_PROFILE_SUPERUSER));};
 
-    inline bool isFeature(unsigned short X) const
-        {return isProfiled() && (profile.features & X);};
+    inline bool has_feature(unsigned short X) const
+        {return is_profiled() && (profile.features & X);};
 };
 
 /**
