@@ -103,13 +103,20 @@ public:
 	static void default_action(context_t ctx, event_t ev);
 	static void automatic_action(context_t ctx);
 
+	static bool publish(context_t ctx, const char *uri, const char *contact, const char *event, const char *duration, const char *type, const char *body);
+
 	static event_t get_event(context_t ctx, timeout_t timeout);
+	static void call_reference(context_t ctx, call_t cid, void *route);
+	static void free_message_request(context_t ctx, voip::msg_t msg);
 	static void release_event(event_t ev);
 
 	static bool listen(context_t ctx, int proto = IPPROTO_UDP, const char *iface = NULL, unsigned port = 5060, bool tls = false);
 	static void create(context_t *ctx, const char *agent, int family = AF_INET);
 	static void release(context_t ctx);
 	static void show(msg_t msg);
+	static void header(msg_t msg, const char *key, const char *value);
+	static void attach(msg_t msg, const char *type, const char *body);
+	static void attach(msg_t msg, const char *type, const char *body, size_t size);
 
 	static void server_allows(voip::msg_t msg);
 	static void server_accepts(voip::msg_t msg);
@@ -129,6 +136,8 @@ public:
 #define SESSION_EVENT   "event"
 #endif
 
+#define	SDP_BODY	"application/sdp"
+
 END_NAMESPACE
 
 #endif
