@@ -217,7 +217,7 @@ public:
     static void cleanup(time_t period);
 };
 
-class __LOCAL stack : private service::callback, private mapped_array<MappedCall>, public OrderedIndex
+class __LOCAL stack : public service::callback, private mapped_array<MappedCall>, public OrderedIndex
 {
 private:
     friend class proxy;
@@ -251,7 +251,9 @@ private:
     {
     public:
         registry::mapped *reg;
-        int cid, tid, did;
+        voip::call_t cid;
+        voip::tid_t tid;
+        voip::did_t did;
         voip::context_t context;
         time_t activates;
         uint32_t sequence;
@@ -287,7 +289,7 @@ private:
     class __LOCAL segment : public OrderedObject
     {
     public:
-        segment(call *cr, int cid, int did = -1, int tid = 0);
+        segment(voip::context_t context, call *cr, voip::call_t cid, voip::did_t did = -1, voip::tid_t tid = 0);
 
         static void *operator new(size_t size);
         static void operator delete(void *obj);
@@ -336,7 +338,6 @@ private:
         void joinLocked(session *s);
         void cancelLocked(void);
         cdr *log(void);
-        void refer(thread *thread, session *s);
         void bye(thread *thread, session *s);
         void set(state_t state, char id, const char *text);
 
@@ -394,11 +395,6 @@ private:
     unsigned invite_expires;
 
 public:
-    voip::context_t out_context;
-    voip::context_t tcp_context;
-    voip::context_t udp_context;
-    voip::context_t tls_context;
-
     static stack sip;
 
     class __LOCAL subnet : public cidr
@@ -438,8 +434,8 @@ public:
 
     static const char *getScheme(void);
     static void getInterface(struct sockaddr *iface, struct sockaddr *dest);
-    static session *create(int cid, int did, int tid);
-    static session *create(call *cr, int cid);
+    static session *create(voip::context_t context, voip::call_t cid, voip::did_t did, voip::tid_t tid);
+    static session *create(voip::context_t context, call *cr, voip::call_t cid);
     static void destroy(session *s);
     static void destroy(call *cr);
     static void disjoin(call *cr);
@@ -580,7 +576,7 @@ private:
     friend class stack;
     friend class stack::call;
 
-    unsigned instance;
+    const char *instance;
     unsigned extension;
     stack::subnet *access;
     char network[MAX_NETWORK_SIZE];
@@ -620,7 +616,7 @@ private:
 
     enum {CALL, MESSAGE, REGISTRAR, NONE} authorizing;
 
-    thread(voip::context_t ctx);
+    thread(voip::context_t ctx, const char *tag);
 
     static void wait(unsigned count);
 
