@@ -131,16 +131,20 @@ cidr(acl, ifaddr(addr, id), id)
     }
 }
 
-stack::segment::segment(call *cr, int cid, int did, int tid) : OrderedObject()
+stack::segment::segment(voip::context_t context, call *cr, int cid, int did, int tid) : OrderedObject()
 {
     assert(cr != NULL);
     assert(cid > 0);
 
     time_t now;
 
+    if(!context)
+        context = stack::sip.out_context;
+
     ++cr->count;
     time(&now);
     enlist(&(cr->segments));
+    sid.context = context;
     sid.enlist(&hash[cid % keysize]);
     sid.sequence = (uint32_t)now;
     sid.sequence &= 0xffffffffl;
@@ -261,9 +265,6 @@ void stack::background::run(void)
             Conditional::unlock();
         }
         messages::automatic();
-        EXOSIP_LOCK
-        eXosip_automatic_action(EXOSIP_CONTEXT);
-        EXOSIP_UNLOCK
     }
 }
 
@@ -455,31 +456,22 @@ void stack::refer(session *source, eXosip_event_t *sevent)
 
     osip_message_header_get_byname(sevent->request, "Refer-To", 0, &header);
     if(!header || !header->hvalue)
-        goto norefer;
+        goto failed;
 
     did = getDialog(target);
-    if(did < 1) {
-norefer:
-        EXOSIP_LOCK
+    if(did < 1)
         goto failed;
-    }
 
-    EXOSIP_LOCK
-    eXosip_call_build_refer(OPTION_CONTEXT did, header->hvalue, &msg);
-    if(!msg) {
+    if(!voip::make_dialog_refer(target->context, did, header->hvalue, &msg)) {
 failed:
-        eXosip_call_build_answer(OPTION_CONTEXT sevent->tid, SIP_SERVICE_UNAVAILABLE, &msg);
-        if(msg)
-            eXosip_call_send_answer(OPTION_CONTEXT sevent->tid, SIP_SERVICE_UNAVAILABLE, msg);
-        EXOSIP_UNLOCK
+        voip::send_answer_response(source->context, sevent->tid, SIP_SERVICE_UNAVAILABLE, NULL);
         return;
     }
     voip::server_allows(msg);
-    osip_message_set_header(msg, "Referred-By", source->identity);
-    eXosip_call_send_request(OPTION_CONTEXT did, msg);
+    voip::header(msg, "Referred-By", source->identity);
+    voip::send_dialog_message(target->context, did, msg);
     target->state = session::REFER;
     target->tid = sevent->tid;
-    EXOSIP_UNLOCK
 }
 
 void stack::infomsg(session *source, eXosip_event_t *sevent)
@@ -508,21 +500,15 @@ void stack::infomsg(session *source, eXosip_event_t *sevent)
         return;
 
     osip_message_get_body(sevent->request, 0, &body);
-    EXOSIP_LOCK
-    eXosip_call_build_info(OPTION_CONTEXT did, &msg);
-    if(!msg) {
-        EXOSIP_UNLOCK
+    if(!voip::make_dialog_info(target->context, did, &msg))
         return;
-    }
     if(ct->subtype)
         snprintf(type, sizeof(type), "%s/%s", ct->type, ct->subtype);
     else
         snprintf(type, sizeof(type), "%s", ct->type);
-    osip_message_set_content_type(msg, type);
-    osip_message_set_body(msg, body->body, strlen(body->body));
+    voip::attach(msg, type, body->body);
     voip::server_allows(msg);
-    eXosip_call_send_request(OPTION_CONTEXT did, msg);
-    EXOSIP_UNLOCK
+    voip::send_dialog_message(target->context, did, msg);
 }
 
 void stack::disjoin(call *cr)
@@ -648,19 +634,19 @@ void stack::getInterface(struct sockaddr *iface, struct sockaddr *dest)
     }
 }
 
-stack::session *stack::create(call *cr, int cid)
+stack::session *stack::create(voip::context_t context, call *cr, voip::call_t cid)
 {
     assert(cr != NULL);
     assert(cid > 0);
 
     locking.exclusive();
-    segment *sp = new segment(cr, cid);
+    segment *sp = new segment(context, cr, cid);
     ++cr->invited;
     locking.share();
     return &sp->sid;
 }
 
-stack::session *stack::create(int cid, int did, int tid)
+stack::session *stack::create(voip::context_t context, voip::call_t cid, voip::did_t did, voip::tid_t tid)
 {
     assert(cid > 0);
 
@@ -674,7 +660,7 @@ stack::session *stack::create(int cid, int did, int tid)
 
     locking.modify();
     cr = new call;
-    sp = new segment(cr, cid, did, tid);    // after count set to 0!
+    sp = new segment(context, cr, cid, did, tid);    // after count set to 0!
     cr->source = &(sp->sid);
     cr->map = map;
 
@@ -722,8 +708,7 @@ void stack::start(service *cfg)
         iface = sip_iface;
 
     thread *thr;
-    shell::log(DEBUG1, "starting sip stack v%d; %d maps and %d threads at priority %d",
-        ver, mapped_calls, threading, priority);
+    shell::log(DEBUG1, "starting sip stack v%d; %d maps", ver, mapped_calls);
 
     tcp_context = udp_context = tls_context = out_context = NULL;
 
@@ -792,7 +777,7 @@ void stack::start(service *cfg)
             shell::log(shell::FAIL, "cannot listen port %u for udp", sip_port);
         else
             shell::log(shell::NOTIFY, "listening port %u for udp", sip_port);
-        thr = new thread(udp_context);
+        thr = new thread(udp_context, "udp");
         thr->start(priority);
     }
 
@@ -801,7 +786,7 @@ void stack::start(service *cfg)
         if(!voip::listen(tcp_context, IPPROTO_TCP, iface, sip_port))
             shell::log(shell::FAIL, "cannot listen port %u for tcp", sip_port);
         shell::log(shell::NOTIFY, "listening port %u for tcp", sip_port);
-        thr = new thread(tcp_context);
+        thr = new thread(tcp_context, "tcp");
         thr->start(priority);
 
     }
@@ -811,7 +796,7 @@ void stack::start(service *cfg)
         if(!voip::listen(tls_context, IPPROTO_TCP, iface, sip_port, true))
             shell::log(shell::FAIL, "cannot listen port %u for tls", sip_port + 1);
         shell::log(shell::NOTIFY, "listening port %u for tls", sip_port + 1);
-        thr = new thread(tls_context);
+        thr = new thread(tls_context, "tls");
         thr->start(priority);
     }
 
@@ -1217,23 +1202,23 @@ void stack::divert(stack::call *call, osip_message_t *invite)
 
     if(String::equal(call->diverting, "all")) {
         snprintf(touri, sizeof(touri), "<%s>;reason=unconditional", route);
-        osip_message_set_header(invite, "Diversion", touri);
+        voip::header(invite, "Diversion", touri);
     }
     else if(String::equal(call->diverting, "na")) {
         snprintf(touri, sizeof(touri), "<%s>;reason=no-answer", route);
-        osip_message_set_header(invite, "Diversion", touri);
+        voip::header(invite, "Diversion", touri);
     }
     else if(String::equal(call->diverting, "busy")) {
         snprintf(touri, sizeof(touri), "<%s>;reason=user-busy", route);
-        osip_message_set_header(invite, "Diversion", touri);
+        voip::header(invite, "Diversion", touri);
     }
     else if(String::equal(call->diverting, "dnd")) {
         snprintf(touri, sizeof(touri), "<%s>;reason=do-not-disturb", route);
-        osip_message_set_header(invite, "Diversion", touri);
+        voip::header(invite, "Diversion", touri);
     }
     else if(String::equal(call->diverting, "away")) {
         snprintf(touri, sizeof(touri), "<%s>;reason=away", route);
-        osip_message_set_header(invite, "Diversion", touri);
+        voip::header(invite, "Diversion", touri);
     }
 }
 
@@ -1350,10 +1335,8 @@ int stack::inviteRemote(stack::session *s, const char *uri_target, const char *d
 
     invite = NULL;
 
-    EXOSIP_LOCK
-    if(eXosip_call_build_initial_invite(OPTION_CONTEXT &invite, touri, s->from, rp, call->subject)) {
+    if(!voip::make_invite_request(context, touri, s->from, call->subject, &invite, rp)) { 
         shell::log(shell::ERR, "cannot invite %s; build failed", uri_target);
-        EXOSIP_UNLOCK
         return icount;
     }
 
@@ -1390,16 +1373,16 @@ int stack::inviteRemote(stack::session *s, const char *uri_target, const char *d
             ",nonce=\"%s\""
             ",algorithm=%s"
             ,s->reg->userid, registry::getRealm(), req, *response, *once, registry::getDigest());
-        osip_message_set_header(invite, AUTHORIZATION, authbuf);
+        voip::header(invite, AUTHORIZATION, authbuf);
         delete[] authbuf;
         osip_free(req);
     }
     else
-        osip_message_set_header(invite, P_SIPWITCH_NODE, "no");
+        voip::header(invite, P_SIPWITCH_NODE, "no");
 
     if(call->expires) {
         snprintf(expheader, sizeof(expheader), "%ld", (long)(call->expires - now));
-        osip_message_set_header(invite, SESSION_EXPIRES, expheader);
+        voip::header(invite, SESSION_EXPIRES, expheader);
     }
 
     char sdp[MAX_SDP_BUFFER];
@@ -1407,29 +1390,26 @@ int stack::inviteRemote(stack::session *s, const char *uri_target, const char *d
 
     if(media::invite(s, network, &nat, sdp) == NULL) {
         shell::log(shell::ERR, "cannot assign media proxy for %s", uri_target);
-        EXOSIP_UNLOCK
+        voip::free_message_request(context, invite);
         return icount;
     }
 
-    osip_message_set_body(invite, sdp, strlen(sdp));
-    osip_message_set_content_type(invite, "application/sdp");
+    voip::attach(invite, SDP_BODY, sdp);
     stack::siplog(invite);
-    cid = eXosip_call_send_initial_invite(OPTION_CONTEXT invite);
+    cid = voip::send_invite_request(context, invite);
     if(cid > 0) {
         snprintf(seqid, sizeof(seqid), "%08x-%d", s->sequence, s->cid);
         uri::publish(call->request, route, seqid, sizeof(route));
-        eXosip_call_set_reference(OPTION_CONTEXT cid, route);
+        voip::call_reference(context, cid, route);
         ++icount;
     }
     else {
         media::release(&nat);
         shell::log(shell::ERR, "invite failed for %s", uri_target);
-        EXOSIP_UNLOCK
         return icount;
     }
 
-    EXOSIP_UNLOCK
-    invited = stack::create(call, cid);
+    invited = stack::create(context, call, cid);
     registry::incUse(NULL, stats::OUTGOING);
     String::set(invited->identity, sizeof(invited->identity), uri_target);
     String::set(invited->display, sizeof(invited->display), username);
@@ -1590,11 +1570,10 @@ int stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t de
         route[0] = '<';
         String::add(route, sizeof(route), ";lr>");
 
-        EXOSIP_LOCK
-        if(eXosip_call_build_initial_invite(OPTION_CONTEXT &invite, touri, s->from, route, call->subject)) {
+        if(!voip::make_invite_request(tp->context, touri, s->from, call->subject, &invite, route)) { 
             stack::sipPublish(&tp->address, route, NULL, sizeof(route));
             shell::log(shell::ERR, "cannot invite %s; build failed", route);
-            goto unlock;
+            goto next;
         }
 
         // if not routing, then separate to from request-uri for forwarding
@@ -1617,36 +1596,34 @@ int stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t de
 
         if(call->expires) {
             snprintf(expheader, sizeof(expheader), "%ld", (long)(call->expires - now));
-            osip_message_set_header(invite, SESSION_EXPIRES, expheader);
+            voip::header(invite, SESSION_EXPIRES, expheader);
         }
 
         nat = NULL;
         if(media::invite(s, tp->network, &nat, sdp) == NULL) {
             stack::sipPublish(&tp->address, route, NULL, sizeof(route));
             shell::log(shell::ERR, "no media proxy available for %s", route);
-            goto unlock;
+            voip::free_message_request(tp->context, invite);
+            goto next;
         }
 
-        osip_message_set_body(invite, sdp, strlen(sdp));
-        osip_message_set_content_type(invite, "application/sdp");
+        voip::attach(invite, SDP_BODY, sdp);
         stack::siplog(invite);
-        cid = eXosip_call_send_initial_invite(OPTION_CONTEXT invite);
+        cid = voip::send_invite_request(tp->context, invite);
         if(cid > 0) {
             snprintf(seqid, sizeof(seqid), "%08x-%d", s->sequence, s->cid);
             stack::sipAddress((struct sockaddr_internet *)&tp->peering, route, seqid, sizeof(route));
-            eXosip_call_set_reference(OPTION_CONTEXT cid, route);
+            voip::call_reference(tp->context, cid, route);
             ++icount;
         }
         else {
             media::release(&nat);
             stack::sipPublish(&tp->address, route, NULL, sizeof(route));
             shell::log(shell::ERR, "invite failed for %s", route);
-            goto unlock;
+            goto next;
         }
 
-        EXOSIP_UNLOCK
-
-        invited = stack::create(call, cid);
+        invited = stack::create(tp->context, call, cid);
 
         String::set(invited->network, sizeof(invited->network), tp->network);
         invited->peering = tp->peering;
@@ -1681,9 +1658,7 @@ int stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t de
         default:
             shell::debug(3, "inviting %s\n", route);
         }
-        goto next;
-unlock:
-        EXOSIP_UNLOCK
+
 next:
         tp.next();
     }
