@@ -42,7 +42,7 @@ static char *remove_quotes(char *c)
     return o;
 }
 
-thread::thread(voip::context_t ctx) : DetachedThread(stack::sip.stacksize)
+thread::thread(voip::context_t ctx, const char *tag) : DetachedThread(stack::sip.stacksize)
 {
     to = NULL;
     from = NULL;
@@ -50,6 +50,7 @@ thread::thread(voip::context_t ctx) : DetachedThread(stack::sip.stacksize)
     routed = NULL;
     reginfo = NULL;
     session = NULL;
+    instance = tag;
     context = ctx;
 }
 
@@ -1176,14 +1177,10 @@ bool thread::authenticate(stack::session *s)
     if(!sip_realm || !*sip_realm || !userid || !secret || !*userid)
         return false;
 
-    EXOSIP_LOCK
-    eXosip_add_authentication_info(OPTION_CONTEXT userid, userid, secret, NULL, sip_realm);
-    eXosip_automatic_action(EXOSIP_CONTEXT);
-    EXOSIP_UNLOCK
+    voip::add_authentication(context, userid, secret, sip_realm, true);
     return true;
 }
 
-
 bool thread::authenticate(void)
 {
     osip_authorization_t *auth = NULL;
@@ -1311,7 +1308,7 @@ void thread::challenge(void)
     case REGISTRAR:
     case MESSAGE:
         if(voip::make_response_message(context, sevent->tid, SIP_UNAUTHORIZED, &reply)) {
-            osip_message_set_header(reply, WWW_AUTHENTICATE, buffer);
+            voip::header(reply, WWW_AUTHENTICATE, buffer);
             voip::server_allows(reply);
             stack::siplog(reply);
             voip::send_response_message(context, sevent->tid, SIP_UNAUTHORIZED, reply);
@@ -1319,7 +1316,7 @@ void thread::challenge(void)
         break;
     case CALL:
         if(voip::make_answer_response(context, sevent->tid, SIP_UNAUTHORIZED, & reply)) {
-            osip_message_set_header(reply, WWW_AUTHENTICATE, buffer);
+            voip::header(reply, WWW_AUTHENTICATE, buffer);
             voip::server_allows(reply);
             stack::siplog(reply);
             voip::send_answer_response(context, sevent->tid, SIP_UNAUTHORIZED, reply);
@@ -1774,13 +1771,13 @@ void thread::expiration(void)
 
 void thread::run(void)
 {
+    time_t current, prior = 0;
     osip_body_t *body;
 
-    instance = ++startup_count;
-    shell::log(DEBUG1, "starting event thread %d", instance);
+    ++startup_count;
+    shell::log(DEBUG1, "starting event thread %s", instance);
 
     for(;;) {
-        assert(instance > 0);
         assert(reginfo == NULL);
         assert(dialed.keys == NULL);
         assert(routed == NULL);
@@ -1802,19 +1799,23 @@ void thread::run(void)
         via_header = NULL;
 
         if(shutdown_flag) {
-            shell::log(DEBUG1, "stopping event thread %d", instance);
+            shell::log(DEBUG1, "stopping event thread %s", instance);
             voip::release(context);
             ++shutdown_count;
             return; // exits thread...
         }
 
-        if(!sevent) {
+        time(&current);
+        if(current != prior) {
+            prior = current;
             voip::automatic_action(context);
-            continue;
         }
 
+        if(!sevent)
+            continue;
+
         ++active_count;
-        shell::debug(2, "sip: event %d; cid=%d, did=%d, instance=%d",
+        shell::debug(2, "sip: event %d; cid=%d, did=%d, instance=%s",
             sevent->type, sevent->cid, sevent->did, instance);
 
         switch(sevent->type) {
@@ -2033,7 +2034,7 @@ void thread::run(void)
             if(sevent->cid < 1)
                 break;
             expiration();
-            session = stack::create(sevent->cid, sevent->did, sevent->tid);
+            session = stack::create(context, sevent->cid, sevent->did, sevent->tid);
             if(!session) {
                 send_reply(SIP_TEMPORARILY_UNAVAILABLE);
                 break;
